@@ -1,7 +1,8 @@
 /*
  *  add.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -114,7 +115,7 @@ int _alpm_add_prepare(pmtrans_t *trans, pmdb_t *db, alpm_list_t **data)
 
 		/* look for unsatisfied dependencies */
 		_alpm_log(PM_LOG_DEBUG, "looking for unsatisfied dependencies\n");
-		lp = alpm_checkdeps(db, 1, NULL, trans->packages);
+		lp = alpm_checkdeps(_alpm_db_get_pkgcache(db), 1, NULL, trans->packages);
 		if(lp != NULL) {
 			if(data) {
 				*data = lp;
@@ -166,7 +167,7 @@ int _alpm_add_prepare(pmtrans_t *trans, pmdb_t *db, alpm_list_t **data)
 		EVENT(trans, PM_TRANS_EVT_FILECONFLICTS_START, NULL, NULL);
 
 		_alpm_log(PM_LOG_DEBUG, "looking for file conflicts\n");
-		lp = _alpm_db_find_fileconflicts(db, trans, handle->root);
+		lp = _alpm_db_find_fileconflicts(db, trans, trans->packages, NULL);
 		if(lp != NULL) {
 			if(data) {
 				*data = lp;
@@ -292,12 +293,12 @@ static int extract_single_file(struct archive *archive,
 		/* the install script goes inside the db */
 		snprintf(filename, PATH_MAX, "%s%s-%s/install", db->path,
 				newpkg->name, newpkg->version);
-		archive_entry_set_mode(entry, 0644);
+		archive_entry_set_perm(entry, 0644);
 	} else if(strcmp(entryname, ".CHANGELOG") == 0) {
 		/* the changelog goes inside the db */
 		snprintf(filename, PATH_MAX, "%s%s-%s/changelog", db->path,
 				newpkg->name, newpkg->version);
-		archive_entry_set_mode(entry, 0644);
+		archive_entry_set_perm(entry, 0644);
 	} else if(*entryname == '.') {
 		/* for now, ignore all files starting with '.' that haven't
 		 * already been handled (for future possibilities) */
@@ -352,28 +353,30 @@ static int extract_single_file(struct archive *archive,
 	if(_alpm_lstat(filename, &lsbuf) != 0 || stat(filename, &sbuf) != 0) {
 		/* cases 1,2,3: couldn't stat an existing file, skip all backup checks */
 	} else {
-		if(S_ISDIR(lsbuf.st_mode) && S_ISDIR(entrymode)) {
-			/* case 12: existing dir, ignore it */
-			if(lsbuf.st_mode != entrymode) {
-				/* if filesystem perms are different than pkg perms, warn user */
-				int mask = 07777;
-				_alpm_log(PM_LOG_WARNING, _("directory permissions differ on %s\n"
-							"filesystem: %o  package: %o\n"), entryname, lsbuf.st_mode & mask,
-						entrymode & mask);
-				alpm_logaction("warning: directory permissions differ on %s\n"
+		if(S_ISDIR(lsbuf.st_mode)) {
+			if(S_ISDIR(entrymode)) {
+				/* case 12: existing dir, ignore it */
+				if(lsbuf.st_mode != entrymode) {
+					/* if filesystem perms are different than pkg perms, warn user */
+					int mask = 07777;
+					_alpm_log(PM_LOG_WARNING, _("directory permissions differ on %s\n"
+								"filesystem: %o  package: %o\n"), entryname, lsbuf.st_mode & mask,
+							entrymode & mask);
+					alpm_logaction("warning: directory permissions differ on %s\n"
 							"filesystem: %o  package: %o\n", entryname, lsbuf.st_mode & mask,
-						entrymode & mask);
+							entrymode & mask);
+				}
+				_alpm_log(PM_LOG_DEBUG, "extract: skipping dir extraction of %s\n",
+						entryname);
+				archive_read_data_skip(archive);
+				return(0);
+			} else {
+				/* case 10/11: trying to overwrite dir with file/symlink, don't allow it */
+				_alpm_log(PM_LOG_ERROR, _("extract: not overwriting dir with file %s\n"),
+						entryname);
+				archive_read_data_skip(archive);
+				return(1);
 			}
-			_alpm_log(PM_LOG_DEBUG, "extract: skipping dir extraction of %s\n",
-					entryname);
-			archive_read_data_skip(archive);
-			return(0);
-		} else if(S_ISDIR(lsbuf.st_mode) && S_ISLNK(entrymode)) {
-			/* case 11: existing dir, symlink in package, ignore it */
-			_alpm_log(PM_LOG_DEBUG, "extract: skipping symlink extraction of %s\n",
-					entryname);
-			archive_read_data_skip(archive);
-			return(0);
 		} else if(S_ISLNK(lsbuf.st_mode) && S_ISDIR(entrymode)) {
 			/* case 9: existing symlink, dir in package */
 			if(S_ISDIR(sbuf.st_mode)) {
@@ -389,12 +392,6 @@ static int extract_single_file(struct archive *archive,
 				archive_read_data_skip(archive);
 				return(1);
 			}
-		} else if(S_ISDIR(lsbuf.st_mode) && S_ISREG(entrymode)) {
-			/* case 10: trying to overwrite dir tree with file, don't allow it */
-			_alpm_log(PM_LOG_ERROR, _("extract: not overwriting dir with file %s\n"),
-					entryname);
-			archive_read_data_skip(archive);
-			return(1);
 		} else if(S_ISREG(lsbuf.st_mode) && S_ISDIR(entrymode)) {
 			/* case 6: trying to overwrite file with dir */
 			_alpm_log(PM_LOG_DEBUG, "extract: overwriting file with dir %s\n",
@@ -456,8 +453,8 @@ static int extract_single_file(struct archive *archive,
 			return(1);
 		}
 
-		hash_local = alpm_get_md5sum(filename);
-		hash_pkg = alpm_get_md5sum(checkfile);
+		hash_local = alpm_compute_md5sum(filename);
+		hash_pkg = alpm_compute_md5sum(checkfile);
 
 		/* append the new md5 hash to it's respective entry
 		 * in newpkg's backup (it will be the new orginal) */
@@ -485,9 +482,9 @@ static int extract_single_file(struct archive *archive,
 		_alpm_log(PM_LOG_DEBUG, "original: %s\n", hash_orig);
 
 		if(!oldpkg) {
-			/* looks like we have a local file that has a different hash as the
-			 * file in the package, move it to a .pacorig */
 			if(strcmp(hash_local, hash_pkg) != 0) {
+				/* looks like we have a local file that has a different hash as the
+				 * file in the package, move it to a .pacorig */
 				char newpath[PATH_MAX];
 				snprintf(newpath, PATH_MAX, "%s.pacorig", filename);
 
@@ -511,6 +508,9 @@ static int extract_single_file(struct archive *archive,
 						alpm_logaction("warning: %s saved as %s\n", filename, newpath);
 					}
 				}
+			} else {
+				/* local file is identical to pkg one, so just remove pkg one */
+				unlink(checkfile);
 			}
 		} else if(hash_orig) {
 			/* the fun part */
@@ -618,7 +618,7 @@ static int extract_single_file(struct archive *archive,
 			}
 			_alpm_log(PM_LOG_DEBUG, "appending backup entry for %s\n", filename);
 
-			hash = alpm_get_md5sum(filename);
+			hash = alpm_compute_md5sum(filename);
 			MALLOC(backup, backup_len, RET_ERR(PM_ERR_MEMORY, -1));
 
 			sprintf(backup, "%s\t%s", oldbackup, hash);
@@ -650,17 +650,18 @@ static int commit_single_pkg(pmpkg_t *newpkg, int pkg_current, int pkg_count,
 	if(local) {
 		is_upgrade = 1;
 
-		EVENT(trans, PM_TRANS_EVT_UPGRADE_START, newpkg, NULL);
-		_alpm_log(PM_LOG_DEBUG, "upgrading package %s-%s\n",
-				newpkg->name, newpkg->version);
-
 		/* we'll need to save some record for backup checks later */
 		oldpkg = _alpm_pkg_dup(local);
 		/* make sure all infos are loaded because the database entry
 		 * will be removed soon */
 		_alpm_db_read(oldpkg->origin_data.db, oldpkg, INFRQ_ALL);
+
+		EVENT(trans, PM_TRANS_EVT_UPGRADE_START, newpkg, oldpkg);
+		_alpm_log(PM_LOG_DEBUG, "upgrading package %s-%s\n",
+				newpkg->name, newpkg->version);
+
 		/* copy over the install reason */
-		newpkg->reason = alpm_pkg_get_reason(local);
+		newpkg->reason = alpm_pkg_get_reason(oldpkg);
 
 		/* pre_upgrade scriptlet */
 		if(alpm_pkg_has_scriptlet(newpkg) && !(trans->flags & PM_TRANS_FLAG_NOSCRIPTLET)) {
@@ -886,7 +887,6 @@ int _alpm_add_commit(pmtrans_t *trans, pmdb_t *db)
 	}
 
 	/* run ldconfig if it exists */
-	_alpm_log(PM_LOG_DEBUG, "running \"ldconfig -r %s\"\n", handle->root);
 	_alpm_ldconfig(handle->root);
 
 	return(0);
