@@ -113,7 +113,58 @@ static int setlastupdate(const pmdb_t *db, time_t time)
 	return(ret);
 }
 
+static int checkdbdir(pmdb_t *db)
+{
+	struct stat buf;
+	char *path = db->path;
+
+	if(stat(path, &buf) != 0) {
+		_alpm_log(PM_LOG_DEBUG, "database dir '%s' does not exist, creating it\n",
+				path);
+		if(_alpm_makepath(path) != 0) {
+			RET_ERR(PM_ERR_SYSTEM, -1);
+		}
+	} else if(!S_ISDIR(buf.st_mode)) {
+		_alpm_log(PM_LOG_WARNING, "removing bogus database: %s\n", path);
+		if(unlink(path) != 0 || _alpm_makepath(path) != 0) {
+			RET_ERR(PM_ERR_SYSTEM, -1);
+		}
+	}
+	return(0);
+}
+
 /** Update a package database
+ *
+ * An update of the package database \a db will be attempted. Unless
+ * \a force is true, the update will only be performed if the remote
+ * database was modified since the last update.
+ *
+ * A transaction is necessary for this operation, in order to obtain a
+ * database lock. During this transaction the front-end will be informed
+ * of the download progress of the database via the download callback.
+ *
+ * Example:
+ * @code
+ * pmdb_t *db;
+ * int result;
+ * db = alpm_list_getdata(alpm_option_get_syncdbs());
+ * if(alpm_trans_init(PM_TRANS_TYPE_SYNC, 0, NULL, NULL, NULL) == 0) {
+ *     result = alpm_db_update(0, db);
+ *     alpm_trans_release();
+ *
+ *     if(result > 0) {
+ *	       printf("Unable to update database: %s\n", alpm_strerrorlast());
+ *     } else if(result < 0) {
+ *         printf("Database already up to date\n");
+ *     } else {
+ *         printf("Database updated\n");
+ *     }
+ * }
+ * @endcode
+ *
+ * @ingroup alpm_databases
+ * @note After a successful update, the \link alpm_db_get_pkgcache()
+ * package cache \endlink will be invalidated
  * @param force if true, then forces the update, otherwise update only in case
  * the database isn't up to date
  * @param db pointer to the package database to update
@@ -122,7 +173,6 @@ static int setlastupdate(const pmdb_t *db, time_t time)
  */
 int SYMEXPORT alpm_db_update(int force, pmdb_t *db)
 {
-	alpm_list_t *lp;
 	char *dbfile, *dbfilepath;
 	time_t newmtime = 0, lastupdate = 0;
 	const char *dbpath;
@@ -176,14 +226,9 @@ int SYMEXPORT alpm_db_update(int force, pmdb_t *db)
 		return(-1);
 	} else {
 		/* remove the old dir */
-		_alpm_log(PM_LOG_DEBUG, "flushing database %s\n", db->path);
-		for(lp = _alpm_db_get_pkgcache(db); lp; lp = lp->next) {
-			pmpkg_t *pkg = lp->data;
-			if(pkg && _alpm_db_remove(db, pkg) == -1) {
-				_alpm_log(PM_LOG_ERROR, _("could not remove database entry %s%s\n"),
-						db->treename, pkg->name);
-				RET_ERR(PM_ERR_DB_REMOVE, -1);
-			}
+		if(_alpm_rmrf(db->path) != 0) {
+			_alpm_log(PM_LOG_ERROR, _("could not remove database %s\n"), db->treename);
+			RET_ERR(PM_ERR_DB_REMOVE, -1);
 		}
 
 		/* Cache needs to be rebuilt */
@@ -195,6 +240,7 @@ int SYMEXPORT alpm_db_update(int force, pmdb_t *db)
 		sprintf(dbfilepath, "%s%s" DBEXT, dbpath, db->treename);
 
 		/* uncompress the sync database */
+		checkdbdir(db);
 		ret = _alpm_unpack(dbfilepath, db->path, NULL);
 		if(ret) {
 			free(dbfilepath);
@@ -214,36 +260,6 @@ int SYMEXPORT alpm_db_update(int force, pmdb_t *db)
 	return(0);
 }
 
-int _alpm_db_open(pmdb_t *db)
-{
-	ALPM_LOG_FUNC;
-
-	if(db == NULL) {
-		RET_ERR(PM_ERR_DB_NULL, -1);
-	}
-
-	_alpm_log(PM_LOG_DEBUG, "opening database from path '%s'\n", db->path);
-	db->handle = opendir(db->path);
-	if(db->handle == NULL) {
-		RET_ERR(PM_ERR_DB_OPEN, -1);
-	}
-
-	return(0);
-}
-
-void _alpm_db_close(pmdb_t *db)
-{
-	ALPM_LOG_FUNC;
-
-	if(db == NULL) {
-		return;
-	}
-
-	if(db->handle) {
-		closedir(db->handle);
-		db->handle = NULL;
-	}
-}
 
 static int splitname(const char *target, pmpkg_t *pkg)
 {
@@ -290,13 +306,17 @@ int _alpm_db_populate(pmdb_t *db)
 	struct dirent *ent = NULL;
 	struct stat sbuf;
 	char path[PATH_MAX];
+	DIR *dbdir;
 
 	ALPM_LOG_FUNC;
 
 	ASSERT(db != NULL, RET_ERR(PM_ERR_DB_NULL, -1));
 
-	rewinddir(db->handle);
-	while((ent = readdir(db->handle)) != NULL) {
+	dbdir = opendir(db->path);
+	if(dbdir == NULL) {
+		return(0);
+	}
+	while((ent = readdir(dbdir)) != NULL) {
 		const char *name = ent->d_name;
 		pmpkg_t *pkg;
 
@@ -311,6 +331,7 @@ int _alpm_db_populate(pmdb_t *db)
 
 		pkg = _alpm_pkg_new();
 		if(pkg == NULL) {
+			closedir(dbdir);
 			return(-1);
 		}
 		/* split the db entry name */
@@ -336,6 +357,7 @@ int _alpm_db_populate(pmdb_t *db)
 		count++;
 	}
 
+	closedir(dbdir);
 	db->pkgcache = alpm_list_msort(db->pkgcache, count, _alpm_pkg_cmp);
 	return(count);
 }
@@ -467,7 +489,7 @@ int _alpm_db_read(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq)
 
 				char first = tolower(line[0]);
 				if(first > 'a' && first < 'z') {
-					struct tm tmp_tm = {0}; //initialize to null incase of failure
+					struct tm tmp_tm = {0}; /* initialize to null in case of failure */
 					setlocale(LC_TIME, "C");
 					strptime(line, "%a %b %e %H:%M:%S %Y", &tmp_tm);
 					info->builddate = mktime(&tmp_tm);
@@ -483,7 +505,7 @@ int _alpm_db_read(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq)
 
 				char first = tolower(line[0]);
 				if(first > 'a' && first < 'z') {
-					struct tm tmp_tm = {0}; //initialize to null incase of failure
+					struct tm tmp_tm = {0}; /* initialize to null in case of failure */
 					setlocale(LC_TIME, "C");
 					strptime(line, "%a %b %e %H:%M:%S %Y", &tmp_tm);
 					info->installdate = mktime(&tmp_tm);
@@ -658,12 +680,16 @@ int _alpm_db_prepare(pmdb_t *db, pmpkg_t *info)
 	int retval = 0;
 	char *pkgpath = NULL;
 
-	oldmask = umask(0000);
+	if(checkdbdir(db) != 0) {
+		return(-1);
+	}
 
+	oldmask = umask(0000);
 	pkgpath = get_pkgpath(db, info);
 
 	if((retval = mkdir(pkgpath, 0755)) != 0) {
-		_alpm_log(PM_LOG_ERROR, _("could not create directory %s: %s\n"), pkgpath, strerror(errno));
+		_alpm_log(PM_LOG_ERROR, _("could not create directory %s: %s\n"),
+				pkgpath, strerror(errno));
 	}
 
 	free(pkgpath);
@@ -761,7 +787,7 @@ int _alpm_db_write(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq)
 			if(info->isize) {
 				/* only write installed size, csize is irrelevant once installed */
 				fprintf(fp, "%%SIZE%%\n"
-								"%ju\n\n", (intmax_t)info->isize);
+								"%jd\n\n", (intmax_t)info->isize);
 			}
 			if(info->reason) {
 				fprintf(fp, "%%REASON%%\n"
@@ -770,11 +796,11 @@ int _alpm_db_write(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq)
 		} else {
 			if(info->size) {
 				fprintf(fp, "%%CSIZE%%\n"
-								"%ju\n\n", (intmax_t)info->size);
+								"%jd\n\n", (intmax_t)info->size);
 			}
 			if(info->isize) {
 				fprintf(fp, "%%ISIZE%%\n"
-								"%ju\n\n", (intmax_t)info->isize);
+								"%jd\n\n", (intmax_t)info->isize);
 			}
 			if(info->md5sum) {
 				fprintf(fp, "%%MD5SUM%%\n"
@@ -826,7 +852,7 @@ int _alpm_db_write(pmdb_t *db, pmpkg_t *info, pmdbinfrq_t inforeq)
 		if(info->depends) {
 			fputs("%DEPENDS%\n", fp);
 			for(lp = info->depends; lp; lp = lp->next) {
-				char *depstring = alpm_dep_get_string(lp->data);
+				char *depstring = alpm_dep_compute_string(lp->data);
 				fprintf(fp, "%s\n", depstring);
 				free(depstring);
 			}
