@@ -1,7 +1,8 @@
 /*
  *  be_package.c
  *
- *  Copyright (c) 2002-2008 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -83,7 +84,7 @@ static int parse_descfile(struct archive *a, pmpkg_t *newpkg)
 			} else if(!strcmp(key, "builddate")) {
 				char first = tolower(ptr[0]);
 				if(first > 'a' && first < 'z') {
-					struct tm tmp_tm = {0}; //initialize to null in case of failure
+					struct tm tmp_tm = {0}; /* initialize to null in case of failure */
 					setlocale(LC_TIME, "C");
 					strptime(ptr, "%a %b %e %H:%M:%S %Y", &tmp_tm);
 					newpkg->builddate = mktime(&tmp_tm);
