@@ -1,7 +1,8 @@
 /*
  *  package.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -422,6 +423,22 @@ alpm_list_t SYMEXPORT *alpm_pkg_get_backup(pmpkg_t *pkg)
 	return pkg->backup;
 }
 
+alpm_list_t SYMEXPORT *alpm_pkg_get_removes(pmpkg_t *pkg)
+{
+	ASSERT(pkg != NULL, return(NULL));
+
+	return(pkg->removes);
+}
+
+pmdb_t SYMEXPORT *alpm_pkg_get_db(pmpkg_t *pkg)
+{
+	/* Sanity checks */
+	ASSERT(pkg != NULL, return(NULL));
+	ASSERT(pkg->origin == PKG_FROM_CACHE, return(NULL));
+
+	return(pkg->origin_data.db);
+}
+
 /**
  * Open a package changelog for reading. Similar to fopen in functionality,
  * except that the returned 'file stream' could really be from an archive
@@ -833,6 +850,7 @@ void _alpm_pkg_free(pmpkg_t *pkg)
 	alpm_list_free_inner(pkg->deltas, (alpm_list_fn_free)_alpm_delta_free);
 	alpm_list_free(pkg->deltas);
 	alpm_list_free(pkg->delta_path);
+	alpm_list_free(pkg->removes);
 
 	if(pkg->origin == PKG_FROM_FILE) {
 		FREE(pkg->origin_data.file);
@@ -840,6 +858,19 @@ void _alpm_pkg_free(pmpkg_t *pkg)
 	FREE(pkg);
 }
 
+/* Free transaction specific fields */
+void _alpm_pkg_free_trans(pmpkg_t *pkg)
+{
+	ALPM_LOG_FUNC;
+
+	if(pkg == NULL) {
+		return;
+	}
+
+	alpm_list_free(pkg->removes);
+	pkg->removes = NULL;
+}
+
 /* Is spkg an upgrade for locapkg? */
 int _alpm_pkg_compare_versions(pmpkg_t *spkg, pmpkg_t *localpkg)
 {
