@@ -1,7 +1,8 @@
 /*
  *  remove.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2006 by David Kimpe <dnaku@frugalware.org>
@@ -49,6 +50,7 @@
 int _alpm_remove_loadtarget(pmtrans_t *trans, pmdb_t *db, char *name)
 {
 	pmpkg_t *info;
+	const char *targ;
 
 	ALPM_LOG_FUNC;
 
@@ -56,25 +58,22 @@ int _alpm_remove_loadtarget(pmtrans_t *trans, pmdb_t *db, char *name)
 	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
 	ASSERT(name != NULL, RET_ERR(PM_ERR_WRONG_ARGS, -1));
 
-	if(_alpm_pkg_find(trans->packages, name)) {
+	targ = strchr(name, '/');
+	if(targ && strncmp(name, "local", 5) == 0) {
+		targ++;
+	} else {
+		targ = name;
+	}
+
+	if(_alpm_pkg_find(trans->packages, targ)) {
 		RET_ERR(PM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
-	if((info = _alpm_db_get_pkgfromcache(db, name)) == NULL) {
-		_alpm_log(PM_LOG_DEBUG, "could not find %s in database\n", name);
+	if((info = _alpm_db_get_pkgfromcache(db, targ)) == NULL) {
+		_alpm_log(PM_LOG_DEBUG, "could not find %s in database\n", targ);
 		RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
 	}
 
-	/* ignore holdpkgs on upgrade */
-	if((trans == handle->trans)
-	    && alpm_list_find_str(handle->holdpkg, info->name)) {
-		int resp = 0;
-		QUESTION(trans, PM_TRANS_CONV_REMOVE_HOLDPKG, info, NULL, NULL, &resp);
-		if(!resp) {
-			RET_ERR(PM_ERR_PKG_HOLD, -1);
-		}
-	}
-
 	_alpm_log(PM_LOG_DEBUG, "adding %s in the targets list\n", info->name);
 	trans->packages = alpm_list_add(trans->packages, _alpm_pkg_dup(info));
 
@@ -104,7 +103,7 @@ static void remove_prepare_cascade(pmtrans_t *trans, pmdb_t *db,
 		}
 		alpm_list_free_inner(lp, (alpm_list_fn_free)_alpm_depmiss_free);
 		alpm_list_free(lp);
-		lp = alpm_checkdeps(db, 1, trans->packages, NULL);
+		lp = alpm_checkdeps(_alpm_db_get_pkgcache(db), 1, trans->packages, NULL);
 	}
 }
 
@@ -134,7 +133,7 @@ static void remove_prepare_keep_needed(pmtrans_t *trans, pmdb_t *db,
 		}
 		alpm_list_free_inner(lp, (alpm_list_fn_free)_alpm_depmiss_free);
 		alpm_list_free(lp);
-		lp = alpm_checkdeps(db, 1, trans->packages, NULL);
+		lp = alpm_checkdeps(_alpm_db_get_pkgcache(db), 1, trans->packages, NULL);
 	}
 }
 
@@ -161,7 +160,7 @@ int _alpm_remove_prepare(pmtrans_t *trans, pmdb_t *db, alpm_list_t **data)
 		EVENT(trans, PM_TRANS_EVT_CHECKDEPS_START, NULL, NULL);
 
 		_alpm_log(PM_LOG_DEBUG, "looking for unsatisfied dependencies\n");
-		lp = alpm_checkdeps(db, 1, trans->packages, NULL);
+		lp = alpm_checkdeps(_alpm_db_get_pkgcache(db), 1, trans->packages, NULL);
 		if(lp != NULL) {
 
 			if(trans->flags & PM_TRANS_FLAG_CASCADE) {
@@ -229,25 +228,18 @@ static int can_remove_file(pmtrans_t *trans, const char *path)
 
 /* Helper function for iterating through a package's file and deleting them
  * Used by _alpm_remove_commit. */
-static void unlink_file(pmpkg_t *info, alpm_list_t *lp, pmtrans_t *trans)
+static void unlink_file(pmpkg_t *info, char *filename, pmtrans_t *trans)
 {
 	struct stat buf;
-	int needbackup = 0;
 	char file[PATH_MAX+1];
 
 	ALPM_LOG_FUNC;
 
-	char *hash = _alpm_needbackup(lp->data, alpm_pkg_get_backup(info));
-	if(hash) {
-		needbackup = 1;
-		FREE(hash);
-	}
-
-	snprintf(file, PATH_MAX, "%s%s", handle->root, (char *)lp->data);
+	snprintf(file, PATH_MAX, "%s%s", handle->root, filename);
 
 	if(trans->type == PM_TRANS_TYPE_REMOVEUPGRADE) {
 		/* check noupgrade */
-		if(alpm_list_find_str(handle->noupgrade, lp->data)) {
+		if(alpm_list_find_str(handle->noupgrade, filename)) {
 			_alpm_log(PM_LOG_DEBUG, "Skipping removal of '%s' due to NoUpgrade\n",
 					file);
 			return;
@@ -278,24 +270,35 @@ static void unlink_file(pmpkg_t *info, alpm_list_t *lp, pmtrans_t *trans)
 			_alpm_log(PM_LOG_DEBUG, "%s is in trans->skip_remove, skipping removal\n",
 					file);
 			return;
-		} else if(needbackup) {
-			/* if the file is flagged, back it up to .pacsave */
-			if(!(trans->flags & PM_TRANS_FLAG_NOSAVE)) {
-				char newpath[PATH_MAX];
-				snprintf(newpath, PATH_MAX, "%s.pacsave", file);
-				rename(file, newpath);
-				_alpm_log(PM_LOG_WARNING, _("%s saved as %s\n"), file, newpath);
-				alpm_logaction("warning: %s saved as %s\n", file, newpath);
-				return;
-			} else {
+		}
+
+		/* if the file needs backup and has been modified, back it up to .pacsave */
+		char *pkghash = _alpm_needbackup(filename, alpm_pkg_get_backup(info));
+		if(pkghash) {
+			if(trans->flags & PM_TRANS_FLAG_NOSAVE) {
 				_alpm_log(PM_LOG_DEBUG, "transaction is set to NOSAVE, not backing up '%s'\n", file);
+				FREE(pkghash);
+			} else {
+				char *filehash = alpm_compute_md5sum(file);
+				int cmp = strcmp(filehash,pkghash);
+				FREE(filehash);
+				FREE(pkghash);
+				if(cmp != 0) {
+					char newpath[PATH_MAX];
+					snprintf(newpath, PATH_MAX, "%s.pacsave", file);
+					rename(file, newpath);
+					_alpm_log(PM_LOG_WARNING, _("%s saved as %s\n"), file, newpath);
+					alpm_logaction("warning: %s saved as %s\n", file, newpath);
+					return;
+				}
 			}
 		}
+
 		_alpm_log(PM_LOG_DEBUG, "unlinking %s\n", file);
 
 		if(unlink(file) == -1) {
 			_alpm_log(PM_LOG_ERROR, _("cannot remove file '%s': %s\n"),
-								(char *)lp->data, strerror(errno));
+								filename, strerror(errno));
 		}
 	}
 }
@@ -360,7 +363,7 @@ int _alpm_remove_commit(pmtrans_t *trans, pmdb_t *db)
 			/* iterate through the list backwards, unlinking files */
 			newfiles = alpm_list_reverse(files);
 			for(lp = newfiles; lp; lp = alpm_list_next(lp)) {
-				unlink_file(info, lp, trans);
+				unlink_file(info, lp->data, trans);
 
 				/* update progress bar after each file */
 				percent = (double)position / (double)filenum;
@@ -405,7 +408,6 @@ int _alpm_remove_commit(pmtrans_t *trans, pmdb_t *db)
 
 	/* run ldconfig if it exists */
 	if(trans->type != PM_TRANS_TYPE_REMOVEUPGRADE) {
-		_alpm_log(PM_LOG_DEBUG, "running \"ldconfig -r %s\"\n", handle->root);
 		_alpm_ldconfig(handle->root);
 	}
 
