@@ -1,7 +1,8 @@
 /*
  *  util.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
  *  Copyright (c) 2006 by David Kimpe <dnaku@frugalware.org>
@@ -36,6 +37,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 /* libarchive */
 #include <archive.h>
@@ -48,6 +50,7 @@
 #include "alpm.h"
 #include "alpm_list.h"
 #include "md5.h"
+#include "handle.h"
 
 #ifndef HAVE_STRSEP
 /* This is a replacement for strsep which is not portable (missing on Solaris).
@@ -242,7 +245,8 @@ char *_alpm_strreplace(const char *str, const char *needle, const char *replace)
 int _alpm_lckmk()
 {
 	int fd;
-	char *dir, *ptr;
+	pid_t pid;
+	char *dir, *ptr, *spid = NULL;
 	const char *file = alpm_option_get_lockfile();
 
 	/* create the dir of the lockfile first */
@@ -256,7 +260,17 @@ int _alpm_lckmk()
 
 	while((fd = open(file, O_WRONLY | O_CREAT | O_EXCL, 0000)) == -1
 			&& errno == EINTR);
-	return(fd > 0 ? fd : -1);
+	if(fd > 0) {
+		pid = getpid();
+		size_t len = snprintf(spid, 0, "%ld\n", (long)pid) + 1;
+		spid = malloc(len);
+		snprintf(spid, len, "%ld\n", (long)pid);
+		while(write(fd, (void *)spid, len) == -1 && errno == EINTR);
+		fsync(fd);
+		free(spid);
+		return(fd);
+	}
+	return(-1);
 }
 
 /* Remove a lock file */
@@ -277,19 +291,21 @@ int _alpm_lckrm()
  * @param archive  the archive to unpack
  * @param prefix   where to extract the files
  * @param fn       a file within the archive to unpack or NULL for all
+ * @return 0 on success, 1 on failure
  */
 int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 {
-	int ret = 1;
+	int ret = 0;
 	mode_t oldmask;
 	struct archive *_archive;
 	struct archive_entry *entry;
-	char expath[PATH_MAX];
+	char cwd[PATH_MAX];
+	int restore_cwd = 0;
 
 	ALPM_LOG_FUNC;
 
 	if((_archive = archive_read_new()) == NULL)
-		RET_ERR(PM_ERR_LIBARCHIVE, -1);
+		RET_ERR(PM_ERR_LIBARCHIVE, 1);
 
 	archive_read_support_compression_all(_archive);
 	archive_read_support_format_all(_archive);
@@ -298,10 +314,25 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 				ARCHIVE_DEFAULT_BYTES_PER_BLOCK) != ARCHIVE_OK) {
 		_alpm_log(PM_LOG_ERROR, _("could not open %s: %s\n"), archive,
 				archive_error_string(_archive));
-		RET_ERR(PM_ERR_PKG_OPEN, -1);
+		RET_ERR(PM_ERR_PKG_OPEN, 1);
 	}
 
 	oldmask = umask(0022);
+
+	/* save the cwd so we can restore it later */
+	if(getcwd(cwd, PATH_MAX) == NULL) {
+		_alpm_log(PM_LOG_ERROR, _("could not get current working directory\n"));
+	} else {
+		restore_cwd = 1;
+	}
+
+	/* just in case our cwd was removed in the upgrade operation */
+	if(chdir(prefix) != 0) {
+		_alpm_log(PM_LOG_ERROR, _("could not change directory to %s (%s)\n"), prefix, strerror(errno));
+		ret = 1;
+		goto cleanup;
+	}
+
 	while(archive_read_next_header(_archive, &entry) == ARCHIVE_OK) {
 		const struct stat *st;
 		const char *entryname; /* the name of the file in the archive */
@@ -310,9 +341,9 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 		entryname = archive_entry_pathname(entry);
 		
 		if(S_ISREG(st->st_mode)) {
-			archive_entry_set_mode(entry, 0644);
+			archive_entry_set_perm(entry, 0644);
 		} else if(S_ISDIR(st->st_mode)) {
-			archive_entry_set_mode(entry, 0755);
+			archive_entry_set_perm(entry, 0755);
 		}
 
 		/* If a specific file was requested skip entries that don't match. */
@@ -326,10 +357,6 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 		}
 
 		/* Extract the archive entry. */
-		ret = 0;
-		snprintf(expath, PATH_MAX, "%s/%s", prefix, entryname);
-		archive_entry_set_pathname(entry, expath);
-
 		int readret = archive_read_extract(_archive, entry, 0);
 		if(readret == ARCHIVE_WARN) {
 			/* operation succeeded but a non-critical error was encountered */
@@ -350,6 +377,9 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 cleanup:
 	umask(oldmask);
 	archive_read_finish(_archive);
+	if(restore_cwd) {
+		chdir(cwd);
+	}
 	return(ret);
 }
 
@@ -427,17 +457,112 @@ int _alpm_logaction(unsigned short usesyslog, FILE *f,
 	return(ret);
 }
 
+int _alpm_run_chroot(const char *root, const char *cmd)
+{
+	char cwd[PATH_MAX];
+	pid_t pid;
+	int restore_cwd = 0;
+	int retval = 0;
+
+	ALPM_LOG_FUNC;
+
+	/* save the cwd so we can restore it later */
+	if(getcwd(cwd, PATH_MAX) == NULL) {
+		_alpm_log(PM_LOG_ERROR, _("could not get current working directory\n"));
+	} else {
+		restore_cwd = 1;
+	}
+
+	/* just in case our cwd was removed in the upgrade operation */
+	if(chdir(root) != 0) {
+		_alpm_log(PM_LOG_ERROR, _("could not change directory to %s (%s)\n"), root, strerror(errno));
+		goto cleanup;
+	}
+
+	_alpm_log(PM_LOG_DEBUG, "executing \"%s\" under chroot \"%s\"\n", cmd, root);
+
+	/* Flush open fds before fork() to avoid cloning buffers */
+	fflush(NULL);
+
+	/* fork- parent and child each have seperate code blocks below */
+	pid = fork();
+	if(pid == -1) {
+		_alpm_log(PM_LOG_ERROR, _("could not fork a new process (%s)\n"), strerror(errno));
+		retval = 1;
+		goto cleanup;
+	}
+
+	if(pid == 0) {
+		FILE *pipe;
+		/* this code runs for the child only (the actual chroot/exec) */
+		_alpm_log(PM_LOG_DEBUG, "chrooting in %s\n", root);
+		if(chroot(root) != 0) {
+			_alpm_log(PM_LOG_ERROR, _("could not change the root directory (%s)\n"),
+					strerror(errno));
+			exit(1);
+		}
+		if(chdir("/") != 0) {
+			_alpm_log(PM_LOG_ERROR, _("could not change directory to / (%s)\n"),
+					strerror(errno));
+			exit(1);
+		}
+		umask(0022);
+		pipe = popen(cmd, "r");
+		if(!pipe) {
+			_alpm_log(PM_LOG_ERROR, _("call to popen failed (%s)"),
+					strerror(errno));
+			exit(1);
+		}
+		while(!feof(pipe)) {
+			char line[PATH_MAX];
+			if(fgets(line, PATH_MAX, pipe) == NULL)
+				break;
+			alpm_logaction("%s", line);
+			EVENT(handle->trans, PM_TRANS_EVT_SCRIPTLET_INFO, line, NULL);
+		}
+		retval = pclose(pipe);
+		exit(WEXITSTATUS(retval));
+	} else {
+		/* this code runs for the parent only (wait on the child) */
+		pid_t retpid;
+		int status;
+		while((retpid = waitpid(pid, &status, 0)) == -1 && errno == EINTR);
+		if(retpid == -1) {
+			_alpm_log(PM_LOG_ERROR, _("call to waitpid failed (%s)\n"),
+			          strerror(errno));
+			retval = 1;
+			goto cleanup;
+		} else {
+			/* check the return status, make sure it is 0 (success) */
+			if(WIFEXITED(status)) {
+				_alpm_log(PM_LOG_DEBUG, "call to waitpid succeeded\n");
+				if(WEXITSTATUS(status) != 0) {
+					_alpm_log(PM_LOG_ERROR, _("command failed to execute correctly\n"));
+					retval = 1;
+				}
+			}
+		}
+	}
+
+cleanup:
+	if(restore_cwd) {
+		chdir(cwd);
+	}
+
+	return(retval);
+}
+
 int _alpm_ldconfig(const char *root)
 {
 	char line[PATH_MAX];
 
+	_alpm_log(PM_LOG_DEBUG, "running ldconfig\n");
+
 	snprintf(line, PATH_MAX, "%setc/ld.so.conf", root);
 	if(access(line, F_OK) == 0) {
 		snprintf(line, PATH_MAX, "%ssbin/ldconfig", root);
 		if(access(line, X_OK) == 0) {
-			char cmd[PATH_MAX];
-			snprintf(cmd, PATH_MAX, "%s -r %s", line, root);
-			system(cmd);
+			_alpm_run_chroot(root, "ldconfig");
 		}
 	}
 
@@ -539,7 +664,7 @@ int _alpm_lstat(const char *path, struct stat *buf)
  * @return the checksum on success, NULL on error
  * @addtogroup alpm_misc
  */
-char SYMEXPORT *alpm_get_md5sum(const char *filename)
+char SYMEXPORT *alpm_compute_md5sum(const char *filename)
 {
 	unsigned char output[16];
 	char *md5sum;
@@ -573,7 +698,7 @@ int _alpm_test_md5sum(const char *filepath, const char *md5sum)
 	char *md5sum2;
 	int ret;
 
-	md5sum2 = alpm_get_md5sum(filepath);
+	md5sum2 = alpm_compute_md5sum(filepath);
 
 	if(md5sum == NULL || md5sum2 == NULL) {
 		ret = -1;
