@@ -9,8 +9,10 @@ p2 = pmpkg("pkg2")
 p2.files = ["dir/symdir/file"]
 self.addpkg(p2)
 
-self.args = "-A %s" % " ".join([p.filename() for p in p1, p2])
+self.args = "-U %s" % " ".join([p.filename() for p in p1, p2])
 
 self.addrule("PACMAN_RETCODE=1")
 self.addrule("!PKG_EXIST=pkg1")
 self.addrule("!PKG_EXIST=pkg2")
+
+self.expectfailure = True
