@@ -1,5 +1,8 @@
 self.description = "Synchronize the local database"
 
+if not "download" in "-lfetch -larchive ":
+	self.option['XferCommand'] = ['/usr/bin/curl %u > %o']
+
 sp1 = pmpkg("spkg1", "1.0-1")
 sp1.depends = ["spkg2"]
 sp2 = pmpkg("spkg2", "2.0-1")
