@@ -6,7 +6,6 @@ sp.conflicts = ["foo"]
 self.addpkg2db("sync", sp)
 
 lp = pmpkg("foo")
-lp.requiredby = ["lp1", "lp2"]
 self.addpkg2db("local", lp)
 
 lp1 = pmpkg("pkg1")
@@ -23,6 +22,4 @@ self.addrule("PACMAN_RETCODE=0")
 self.addrule("!PKG_EXIST=foo")
 self.addrule("PKG_EXIST=bar")
 self.addrule("PKG_EXIST=pkg1")
-self.addrule("PKG_REQUIREDBY=bar|pkg1")
 self.addrule("PKG_EXIST=pkg2")
-self.addrule("PKG_REQUIREDBY=bar|pkg2")
