@@ -1,7 +1,8 @@
 /*
  *  package.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -79,7 +80,7 @@ void dump_pkg_full(pmpkg_t *pkg, int level)
 	/* turn depends list into a text list */
 	for(i = alpm_pkg_get_depends(pkg); i; i = alpm_list_next(i)) {
 		pmdepend_t *dep = (pmdepend_t*)alpm_list_getdata(i);
-		depstrings = alpm_list_add(depstrings, alpm_dep_get_string(dep));
+		depstrings = alpm_list_add(depstrings, alpm_dep_compute_string(dep));
 	}
 
 	if(level>0) {
@@ -176,7 +177,7 @@ void dump_pkg_backups(pmpkg_t *pkg)
 			snprintf(path, PATH_MAX-1, "%s%s", root, str);
 			/* if we find the file, calculate checksums, otherwise it is missing */
 			if(access(path, R_OK) == 0) {
-				char *md5sum = alpm_get_md5sum(path);
+				char *md5sum = alpm_compute_md5sum(path);
 
 				if(md5sum == NULL) {
 					pm_fprintf(stderr, PM_LOG_ERROR,
@@ -205,7 +206,7 @@ void dump_pkg_backups(pmpkg_t *pkg)
 
 /* List all files contained in a package
  */
-void dump_pkg_files(pmpkg_t *pkg)
+void dump_pkg_files(pmpkg_t *pkg, int quiet)
 {
 	const char *pkgname, *root, *filestr;
 	alpm_list_t *i, *pkgfiles;
@@ -216,7 +217,11 @@ void dump_pkg_files(pmpkg_t *pkg)
 
 	for(i = pkgfiles; i; i = alpm_list_next(i)) {
 		filestr = alpm_list_getdata(i);
-		fprintf(stdout, "%s %s%s\n", pkgname, root, filestr);
+		if(!quiet){
+			fprintf(stdout, "%s %s%s\n", pkgname, root, filestr);
+		} else {
+			fprintf(stdout, "%s%s\n", root, filestr);
+		}
 	}
 
 	fflush(stdout);
