@@ -1,7 +1,8 @@
 /*
  *  pacman.c
  *
- *  Copyright (c) 2002-2008 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -65,8 +66,8 @@ static void usage(int op, const char * const myname)
 	/* prefetch some strings for usage below, which moves a lot of calls
 	 * out of gettext. */
 	char const * const str_opt = _("options");
-	char const * const str_file = _("file");
-	char const * const str_pkg = _("package");
+	char const * const str_file = _("file(s)");
+	char const * const str_pkg = _("package(s)");
 	char const * const str_usg = _("usage");
 	char const * const str_opr = _("operation");
 
@@ -103,17 +104,18 @@ static void usage(int op, const char * const myname)
 			printf("%s:  %s {-Q --query} [%s] [%s]\n", str_usg, myname, str_opt, str_pkg);
 			printf("%s:\n", str_opt);
 			printf(_("  -c, --changelog      view the changelog of a package\n"));
-			printf(_("  -d, --deps           list all packages installed as dependencies\n"));
-			printf(_("  -e, --explicit       list all packages explicitly installed\n"));
+			printf(_("  -d, --deps           list packages installed as dependencies [filter]\n"));
+			printf(_("  -e, --explicit       list packages explicitly installed [filter]\n"));
 			printf(_("  -g, --groups         view all members of a package group\n"));
 			printf(_("  -i, --info           view package information (-ii for backup files)\n"));
+			printf(_("  -k, --check          check that the files owned by the package(s) are present\n"));
 			printf(_("  -l, --list           list the contents of the queried package\n"));
-			printf(_("  -m, --foreign        list installed packages not found in sync db(s)\n"));
+			printf(_("  -m, --foreign        list installed packages not found in sync db(s) [filter]\n"));
 			printf(_("  -o, --owns <file>    query the package that owns <file>\n"));
 			printf(_("  -p, --file <package> query a package file instead of the database\n"));
 			printf(_("  -s, --search <regex> search locally-installed packages for matching strings\n"));
-			printf(_("  -t, --unrequired     list all packages not required by any package\n"));
-			printf(_("  -u, --upgrades       list all packages that can be upgraded\n"));
+			printf(_("  -t, --unrequired     list packages not required by any package [filter]\n"));
+			printf(_("  -u, --upgrades       list outdated packages [filter]\n"));
 			printf(_("  -q, --quiet          show less information for query and search\n"));
 		} else if(op == PM_OP_SYNC) {
 			printf("%s:  %s {-S --sync} [%s] [%s]\n", str_usg, myname, str_opt, str_pkg);
@@ -128,7 +130,7 @@ static void usage(int op, const char * const myname)
 			printf(_("  -l, --list <repo>    view a list of packages in a repo\n"));
 			printf(_("  -p, --print-uris     print out URIs for given packages and their dependencies\n"));
 			printf(_("  -s, --search <regex> search remote repositories for matching strings\n"));
-			printf(_("  -u, --sysupgrade     upgrade all packages that are out of date\n"));
+			printf(_("  -u, --sysupgrade     upgrade all outdated packages (-uu enables downgrade)\n"));
 			printf(_("  -w, --downloadonly   download packages but do not install/upgrade anything\n"));
 			printf(_("  -y, --refresh        download fresh package databases from the server\n"));
 			printf(_("      --needed         don't reinstall up to date packages\n"));
@@ -143,6 +145,7 @@ static void usage(int op, const char * const myname)
 		printf(_("      --noprogressbar  do not show a progress bar when downloading files\n"));
 		printf(_("      --noscriptlet    do not execute the install scriptlet if one exists\n"));
 		printf(_("  -v, --verbose        be verbose\n"));
+		printf(_("      --debug          display debug messages\n"));
 		printf(_("  -r, --root <path>    set an alternate installation root\n"));
 		printf(_("  -b, --dbpath <path>  set an alternate database location\n"));
 		printf(_("      --cachedir <dir> set an alternate package cache location\n"));
@@ -155,8 +158,8 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, alpm_version());
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2008 Dan McGee <dan@archlinux.org>\n");
-	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet <jvinet@zeroflux.org>\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2009 Pacman Development Team\n");
+	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
 	         "                       the terms of the GNU General Public License.\n"));
@@ -343,6 +346,7 @@ static int parseargs(int argc, char *argv[])
 		{"help",       no_argument,       0, 'h'},
 		{"info",       no_argument,       0, 'i'},
 		{"dbonly",     no_argument,       0, 'k'},
+		{"check",      no_argument,       0, 'k'},
 		{"list",       no_argument,       0, 'l'},
 		{"foreign",    no_argument,       0, 'm'},
 		{"nosave",     no_argument,       0, 'n'},
@@ -471,14 +475,19 @@ static int parseargs(int argc, char *argv[])
 			case 'g': (config->group)++; break;
 			case 'h': config->help = 1; break;
 			case 'i': (config->op_q_info)++; (config->op_s_info)++; break;
-			case 'k': config->flags |= PM_TRANS_FLAG_DBONLY; break;
+			case 'k':
+				config->flags |= PM_TRANS_FLAG_DBONLY;
+				config->op_q_check = 1;
+				break;
 			case 'l': config->op_q_list = 1; break;
 			case 'm': config->op_q_foreign = 1; break;
 			case 'n': config->flags |= PM_TRANS_FLAG_NOSAVE; break;
 			case 'o': config->op_q_owns = 1; break;
 			case 'p':
 				config->op_q_isfile = 1;
-				config->flags |= PM_TRANS_FLAG_PRINTURIS;
+				config->op_s_printuris = 1;
+				config->flags |= PM_TRANS_FLAG_NOCONFLICTS;
+				config->flags |= PM_TRANS_FLAG_NOLOCK;
 				break;
 			case 'q':
 				config->quiet = 1;
@@ -499,7 +508,7 @@ static int parseargs(int argc, char *argv[])
 				config->op_q_unrequired = 1;
 				break;
 			case 'u':
-				config->op_s_upgrade = 1;
+				(config->op_s_upgrade)++;
 				config->op_q_upgrade = 1;
 				config->flags |= PM_TRANS_FLAG_UNNEEDED;
 				break;
@@ -538,6 +547,11 @@ static int parseargs(int argc, char *argv[])
 	return(0);
 }
 
+/* helper for being used with setrepeatingoption */
+static void option_add_holdpkg(const char *name) {
+	config->holdpkg = alpm_list_add(config->holdpkg, strdup(name));
+}
+
 /* helper for being used with setrepeatingoption */
 static void option_add_syncfirst(const char *name) {
 	config->syncfirst = alpm_list_add(config->syncfirst, strdup(name));
@@ -567,6 +581,118 @@ static void setrepeatingoption(const char *ptr, const char *option,
 	pm_printf(PM_LOG_DEBUG, "config: %s: %s\n", option, p);
 }
 
+static char *get_filename(const char *url) {
+	char *filename = strrchr(url, '/');
+	if(filename != NULL) {
+		filename++;
+	}
+	return(filename);
+}
+
+static char *get_destfile(const char *path, const char *filename) {
+	char *destfile;
+	/* len = localpath len + filename len + null */
+	int len = strlen(path) + strlen(filename) + 1;
+	destfile = calloc(len, sizeof(char));
+	snprintf(destfile, len, "%s%s", path, filename);
+
+	return(destfile);
+}
+
+static char *get_tempfile(const char *path, const char *filename) {
+	char *tempfile;
+	/* len = localpath len + filename len + '.part' len + null */
+	int len = strlen(path) + strlen(filename) + 6;
+	tempfile = calloc(len, sizeof(char));
+	snprintf(tempfile, len, "%s%s.part", path, filename);
+
+	return(tempfile);
+}
+
+/** External fetch callback */
+int download_with_xfercommand(const char *url, const char *localpath,
+		time_t mtimeold, time_t *mtimenew) {
+	int ret = 0;
+	int retval;
+	int usepart = 0;
+	char *ptr1, *ptr2;
+	char origCmd[PATH_MAX];
+	char parsedCmd[PATH_MAX] = "";
+	char cwd[PATH_MAX];
+	char *destfile, *tempfile, *filename;
+
+	if(!config->xfercommand) {
+		return -1;
+	}
+
+	filename = get_filename(url);
+	if(!filename) {
+		return -1;
+	}
+	destfile = get_destfile(localpath, filename);
+	tempfile = get_tempfile(localpath, filename);
+
+	strncpy(origCmd, config->xfercommand, sizeof(origCmd));
+	/* replace all occurrences of %o with fn.part */
+	ptr1 = origCmd;
+	while((ptr2 = strstr(ptr1, "%o"))) {
+		usepart = 1;
+		ptr2[0] = '\0';
+		strcat(parsedCmd, ptr1);
+		strcat(parsedCmd, tempfile);
+		ptr1 = ptr2 + 2;
+	}
+	strcat(parsedCmd, ptr1);
+	/* replace all occurrences of %u with the download URL */
+	strncpy(origCmd, parsedCmd, sizeof(origCmd));
+	parsedCmd[0] = '\0';
+	ptr1 = origCmd;
+	while((ptr2 = strstr(ptr1, "%u"))) {
+		ptr2[0] = '\0';
+		strcat(parsedCmd, ptr1);
+		strcat(parsedCmd, url);
+		ptr1 = ptr2 + 2;
+	}
+	strcat(parsedCmd, ptr1);
+	/* cwd to the download directory */
+	getcwd(cwd, PATH_MAX);
+	if(chdir(localpath)) {
+		pm_printf(PM_LOG_WARNING, "could not chdir to %s\n", localpath);
+		ret = -1;
+		goto cleanup;
+	}
+	/* execute the parsed command via /bin/sh -c */
+	pm_printf(PM_LOG_DEBUG, "running command: %s\n", parsedCmd);
+	retval = system(parsedCmd);
+
+	if(retval == -1) {
+		pm_printf(PM_LOG_WARNING, "running XferCommand: fork failed!\n");
+		ret = -1;
+	} else if(retval != 0) {
+		/* download failed */
+		pm_printf(PM_LOG_DEBUG, "XferCommand command returned non-zero status "
+				"code (%d)\n", retval);
+		ret = -1;
+	} else {
+		/* download was successful */
+		if(usepart) {
+			rename(tempfile, destfile);
+		}
+		ret = 0;
+	}
+
+cleanup:
+	chdir(cwd);
+	if(ret == -1) {
+		/* hack to let an user the time to cancel a download */
+		sleep(2);
+	}
+	free(destfile);
+	free(tempfile);
+
+	return(ret);
+}
+
 /* The real parseconfig. Called with a null section argument by the publicly
  * visible parseconfig so we can recall from within ourself on an include */
 static int _parseconfig(const char *file, const char *givensection,
@@ -701,7 +827,7 @@ static int _parseconfig(const char *file, const char *givensection,
 					} else if(strcmp(key, "IgnoreGroup") == 0) {
 						setrepeatingoption(ptr, "IgnoreGroup", alpm_option_add_ignoregrp);
 					} else if(strcmp(key, "HoldPkg") == 0) {
-						setrepeatingoption(ptr, "HoldPkg", alpm_option_add_holdpkg);
+						setrepeatingoption(ptr, "HoldPkg", option_add_holdpkg);
 					} else if(strcmp(key, "SyncFirst") == 0) {
 						setrepeatingoption(ptr, "SyncFirst", option_add_syncfirst);
 					} else if(strcmp(key, "DBPath") == 0) {
@@ -730,7 +856,8 @@ static int _parseconfig(const char *file, const char *givensection,
 							pm_printf(PM_LOG_DEBUG, "config: logfile: %s\n", ptr);
 						}
 					} else if (strcmp(key, "XferCommand") == 0) {
-						alpm_option_set_xfercommand(ptr);
+						config->xfercommand = strdup(ptr);
+						alpm_option_set_fetchcb(download_with_xfercommand);
 						pm_printf(PM_LOG_DEBUG, "config: xfercommand: %s\n", ptr);
 					} else if (strcmp(key, "CleanMethod") == 0) {
 						if (strcmp(ptr, "KeepInstalled") == 0) {
@@ -893,7 +1020,7 @@ int main(int argc, char *argv[])
 
 #if defined(HAVE_GETEUID) && !defined(CYGWIN)
 	/* check if we have sufficient permission for the requested operation */
-	if(myuid > 0 && needs_transaction()) {
+	if(myuid > 0 && needs_root()) {
 		pm_printf(PM_LOG_ERROR, _("you cannot perform this operation unless you are root.\n"));
 		cleanup(EXIT_FAILURE);
 	}
