@@ -1,7 +1,8 @@
 /*
  *  query.c
  *
- *  Copyright (c) 2002-2008 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -74,7 +75,7 @@ static int query_fileowner(alpm_list_t *targets)
 		struct stat buf;
 		alpm_list_t *i, *j;
 
-		if(stat(filename, &buf) == -1) {
+		if(lstat(filename, &buf) == -1) {
 			pm_fprintf(stderr, PM_LOG_ERROR, _("failed to read file '%s': %s\n"),
 					filename, strerror(errno));
 			ret++;
@@ -103,7 +104,7 @@ static int query_fileowner(alpm_list_t *targets)
 
 		root = alpm_option_get_root();
 
-		for(i = alpm_db_getpkgcache(db_local); i && !found; i = alpm_list_next(i)) {
+		for(i = alpm_db_get_pkgcache(db_local); i && !found; i = alpm_list_next(i)) {
 			pmpkg_t *info = alpm_list_getdata(i);
 
 			for(j = alpm_pkg_get_files(info); j && !found; j = alpm_list_next(j)) {
@@ -125,8 +126,7 @@ static int query_fileowner(alpm_list_t *targets)
 						printf(_("%s is owned by %s %s\n"), filename,
 								alpm_pkg_get_name(info), alpm_pkg_get_version(info));
 					} else {
-						printf("%s %s\n", alpm_pkg_get_name(info),
-								alpm_pkg_get_version(info));
+						printf("%s\n", alpm_pkg_get_name(info));
 					}
 					found = 1;
 				}
@@ -154,7 +154,7 @@ static int query_search(alpm_list_t *targets)
 		searchlist = alpm_db_search(db_local, targets);
 		freelist = 1;
 	} else {
-		searchlist = alpm_db_getpkgcache(db_local);
+		searchlist = alpm_db_get_pkgcache(db_local);
 		freelist = 0;
 	}
 	if(searchlist == NULL) {
@@ -215,7 +215,7 @@ static int query_group(alpm_list_t *targets)
 	char *grpname = NULL;
 	int ret = 0;
 	if(targets == NULL) {
-		for(j = alpm_db_getgrpcache(db_local); j; j = alpm_list_next(j)) {
+		for(j = alpm_db_get_grpcache(db_local); j; j = alpm_list_next(j)) {
 			pmgrp_t *grp = alpm_list_getdata(j);
 			const alpm_list_t *p, *packages;
 			const char *grpname;
@@ -251,24 +251,6 @@ static int query_group(alpm_list_t *targets)
 	return ret;
 }
 
-static int query_upgrades(void)
-{
-	alpm_list_t *syncpkgs = NULL;
-	printf(_("Checking for package upgrades... \n"));
-
-	alpm_list_t *syncdbs = alpm_option_get_syncdbs();
-	if(alpm_sync_sysupgrade(db_local, syncdbs, &syncpkgs) == -1) {
-		return(-1);
-	}
-	if(syncpkgs) {
-		display_synctargets(syncpkgs);
-		return(0);
-	}
-
-	printf(_("no upgrades found.\n"));
-	return(1);
-}
-
 static int is_foreign(pmpkg_t *pkg)
 {
 	const char *pkgname = alpm_pkg_get_name(pkg);
@@ -320,11 +302,67 @@ static int filter(pmpkg_t *pkg)
 	if(config->op_q_unrequired && !is_unrequired(pkg)) {
 		return(0);
 	}
+	/* check if this pkg is outdated */
+	if(config->op_q_upgrade && (alpm_sync_newversion(pkg, alpm_option_get_syncdbs()) == NULL)) {
+		return(0);
+	}
 	return(1);
 }
 
-static void display(pmpkg_t *pkg)
+/* Loop through the packages. For each package,
+ * loop through files to check if they exist. */
+static int check(pmpkg_t *pkg)
 {
+	alpm_list_t *i;
+	const char *root;
+	int allfiles = 0, errors = 0;
+	size_t rootlen;
+	char f[PATH_MAX];
+
+	root = alpm_option_get_root();
+	rootlen = strlen(root);
+	if(rootlen + 1 > PATH_MAX) {
+		/* we are in trouble here */
+		pm_fprintf(stderr, PM_LOG_ERROR, _("root path too long\n"));
+		return(1);
+	}
+	strcpy(f, root);
+
+	const char *pkgname = alpm_pkg_get_name(pkg);
+	for(i = alpm_pkg_get_files(pkg); i; i = alpm_list_next(i)) {
+		struct stat st;
+		const char *path = alpm_list_getdata(i);
+
+		if(rootlen + 1 + strlen(path) > PATH_MAX) {
+			pm_fprintf(stderr, PM_LOG_WARNING, _("file path too long\n"));
+			continue;
+		}
+		strcpy(f + rootlen, path);
+		allfiles++;
+		/* use lstat to prevent errors from symlinks */
+		if(lstat(f, &st) != 0) {
+			if(config->quiet) {
+				printf("%s %s\n", pkgname, f);
+			} else {
+				pm_printf(PM_LOG_WARNING, "%s: missing %s (%s)\n",
+						pkgname, f, strerror(errno));
+			}
+			errors++;
+		}
+	}
+
+	if(!config->quiet) {
+		printf("%s: %d total, %d missing file(s)\n",
+				pkgname, allfiles, errors);
+	}
+
+	return(errors != 0 ? 1 : 0);
+}
+
+static int display(pmpkg_t *pkg)
+{
+	int ret = 0;
+
 	if(config->op_q_info) {
 		if(config->op_q_isfile) {
 			/* omit info that isn't applicable for a file package */
@@ -334,24 +372,30 @@ static void display(pmpkg_t *pkg)
 		}
 	}
 	if(config->op_q_list) {
-		dump_pkg_files(pkg);
+		dump_pkg_files(pkg, config->quiet);
 	}
 	if(config->op_q_changelog) {
 		dump_pkg_changelog(pkg);
 	}
-	if(!config->op_q_info && !config->op_q_list && !config->op_q_changelog) {
+	if(config->op_q_check) {
+		ret = check(pkg);
+	}
+	if(!config->op_q_info && !config->op_q_list
+			&& !config->op_q_changelog && !config->op_q_check) {
 		if (!config->quiet) {
 			printf("%s %s\n", alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg));
 		} else {
 			printf("%s\n", alpm_pkg_get_name(pkg));
 		}
 	}
+	return(ret);
 }
 
 int pacman_query(alpm_list_t *targets)
 {
 	int ret = 0;
 	alpm_list_t *i;
+	pmpkg_t *pkg = NULL;
 
 	/* First: operations that do not require targets */
 
@@ -361,12 +405,6 @@ int pacman_query(alpm_list_t *targets)
 		return(ret);
 	}
 
-	/* check for package upgrades */
-	if(config->op_q_upgrade) {
-		ret = query_upgrades();
-		return(ret);
-	}
-
 	/* looking for groups */
 	if(config->group) {
 		ret = query_group(targets);
@@ -378,12 +416,12 @@ int pacman_query(alpm_list_t *targets)
 		alpm_list_t *sync_dbs = alpm_option_get_syncdbs();
 		if(sync_dbs == NULL || alpm_list_count(sync_dbs) == 0) {
 			pm_printf(PM_LOG_ERROR, _("no usable package repositories configured.\n"));
-			return(-1);
+			return(1);
 		}
 	}
 
 	/* operations on all packages in the local DB
-	 * valid: no-op (plain -Q), list, info
+	 * valid: no-op (plain -Q), list, info, check
 	 * invalid: isfile, owns */
 	if(targets == NULL) {
 		if(config->op_q_isfile || config->op_q_owns) {
@@ -391,13 +429,16 @@ int pacman_query(alpm_list_t *targets)
 			return(1);
 		}
 
-		for(i = alpm_db_getpkgcache(db_local); i; i = alpm_list_next(i)) {
-			pmpkg_t *pkg = alpm_list_getdata(i);
+		for(i = alpm_db_get_pkgcache(db_local); i; i = alpm_list_next(i)) {
+			pkg = alpm_list_getdata(i);
 			if(filter(pkg)) {
-				display(pkg);
+				int value = display(pkg);
+				if(value != 0) {
+					ret = 1;
+				}
 			}
 		}
-		return(0);
+		return(ret);
 	}
 
 	/* Second: operations that require target(s) */
@@ -409,10 +450,9 @@ int pacman_query(alpm_list_t *targets)
 	}
 
 	/* operations on named packages in the local DB
-	 * valid: no-op (plain -Q), list, info */
+	 * valid: no-op (plain -Q), list, info, check */
 	for(i = targets; i; i = alpm_list_next(i)) {
 		char *strname = alpm_list_getdata(i);
-		pmpkg_t *pkg = NULL;
 
 		if(config->op_q_isfile) {
 			alpm_pkg_load(strname, 1, &pkg);
@@ -422,12 +462,15 @@ int pacman_query(alpm_list_t *targets)
 
 		if(pkg == NULL) {
 			pm_fprintf(stderr, PM_LOG_ERROR, _("package \"%s\" not found\n"), strname);
-			ret++;
+			ret = 1;
 			continue;
 		}
 
 		if(filter(pkg)) {
-			display(pkg);
+			int value = display(pkg);
+			if(value != 0) {
+				ret = 1;
+			}
 		}
 
 		if(config->op_q_isfile) {
