@@ -1,7 +1,8 @@
 /*
  *  sync.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -85,7 +86,7 @@ static int sync_cleandb(const char *dbpath, int keep_used) {
 		/* We have a directory that doesn't match any syncdb.
 		 * Ask the user if he wants to remove it. */
 		if(!found) {
-			if(!yesno(1, _("Do you want to remove %s?"), path)) {
+			if(!yesno(_("Do you want to remove %s?"), path)) {
 				continue;
 			}
 
@@ -105,7 +106,7 @@ static int sync_cleandb_all(void) {
 	char newdbpath[PATH_MAX];
 
 	printf(_("Database directory: %s\n"), dbpath);
-	if(!yesno(1, _("Do you want to remove unused repositories?"))) {
+	if(!yesno(_("Do you want to remove unused repositories?"))) {
 		return(0);
 	}
 	/* The sync dbs were previously put in dbpath/, but are now in dbpath/sync/,
@@ -135,12 +136,12 @@ static int sync_cleancache(int level)
 		printf(_("Cache directory: %s\n"), cachedir);
 		switch(config->cleanmethod) {
 			case PM_CLEAN_KEEPINST:
-				if(!yesno(1, _("Do you want to remove uninstalled packages from cache?"))) {
+				if(!yesno(_("Do you want to remove uninstalled packages from cache?"))) {
 					return(0);
 				}
 				break;
 			case PM_CLEAN_KEEPCUR:
-				if(!yesno(1, _("Do you want to remove outdated packages from cache?"))) {
+				if(!yesno(_("Do you want to remove outdated packages from cache?"))) {
 					return(0);
 				}
 				break;
@@ -148,7 +149,7 @@ static int sync_cleancache(int level)
 				/* this should not happen : the config parsing doesn't set any other value */
 				return(1);
 		}
-		printf(_("removing old packages from cache... "));
+		printf(_("removing old packages from cache...\n"));
 
 		dir = opendir(cachedir);
 		if(dir == NULL) {
@@ -171,10 +172,13 @@ static int sync_cleancache(int level)
 			/* build the full filepath */
 			snprintf(path, PATH_MAX, "%s%s", cachedir, ent->d_name);
 
-			/* attempt to load the package, skip file on failures as we may have
+			/* attempt to load the package, prompt removal on failures as we may have
 			 * files here that aren't valid packages. we also don't need a full
 			 * load of the package, just the metadata. */
 			if(alpm_pkg_load(path, 0, &localpkg) != 0 || localpkg == NULL) {
+				if(yesno(_("File %s does not seem to be a valid package, remove it?"), path)) {
+					unlink(path);
+				}
 				continue;
 			}
 			switch(config->cleanmethod) {
@@ -211,14 +215,13 @@ static int sync_cleancache(int level)
 				unlink(path);
 			}
 		}
-		printf(_("done.\n"));
 	} else {
 		/* full cleanup */
 		printf(_("Cache directory: %s\n"), cachedir);
-		if(!yesno(0, _("Do you want to remove ALL packages from cache?"))) {
+		if(!noyes(_("Do you want to remove ALL files from cache?"))) {
 			return(0);
 		}
-		printf(_("removing all packages from cache... "));
+		printf(_("removing all files from cache...\n"));
 
 		if(rmrf(cachedir)) {
 			pm_fprintf(stderr, PM_LOG_ERROR, _("could not remove cache directory\n"));
@@ -229,7 +232,6 @@ static int sync_cleancache(int level)
 			pm_fprintf(stderr, PM_LOG_ERROR, _("could not create new cache directory\n"));
 			return(1);
 		}
-		printf(_("done.\n"));
 	}
 
 	return(0);
@@ -286,7 +288,7 @@ static int sync_search(alpm_list_t *syncs, alpm_list_t *targets)
 			ret = alpm_db_search(db, targets);
 			freelist = 1;
 		} else {
-			ret = alpm_db_getpkgcache(db);
+			ret = alpm_db_get_pkgcache(db);
 			freelist = 0;
 		}
 		if(ret == NULL) {
@@ -371,7 +373,7 @@ static int sync_group(int level, alpm_list_t *syncs, alpm_list_t *targets)
 		for(i = syncs; i; i = alpm_list_next(i)) {
 			pmdb_t *db = alpm_list_getdata(i);
 
-			for(j = alpm_db_getgrpcache(db); j; j = alpm_list_next(j)) {
+			for(j = alpm_db_get_grpcache(db); j; j = alpm_list_next(j)) {
 				pmgrp_t *grp = alpm_list_getdata(j);
 				const char *grpname = alpm_grp_get_name(grp);
 
@@ -425,7 +427,7 @@ static int sync_info(alpm_list_t *syncs, alpm_list_t *targets)
 					return(1);
 				}
 
-				for(k = alpm_db_getpkgcache(db); k; k = alpm_list_next(k)) {
+				for(k = alpm_db_get_pkgcache(db); k; k = alpm_list_next(k)) {
 					pmpkg_t *pkg = alpm_list_getdata(k);
 
 					if(strcmp(alpm_pkg_get_name(pkg), pkgstr) == 0) {
@@ -446,7 +448,7 @@ static int sync_info(alpm_list_t *syncs, alpm_list_t *targets)
 				for(j = syncs; j; j = alpm_list_next(j)) {
 					pmdb_t *db = alpm_list_getdata(j);
 
-					for(k = alpm_db_getpkgcache(db); k; k = alpm_list_next(k)) {
+					for(k = alpm_db_get_pkgcache(db); k; k = alpm_list_next(k)) {
 						pmpkg_t *pkg = alpm_list_getdata(k);
 
 						if(strcmp(alpm_pkg_get_name(pkg), pkgstr) == 0) {
@@ -467,7 +469,7 @@ static int sync_info(alpm_list_t *syncs, alpm_list_t *targets)
 		for(i = syncs; i; i = alpm_list_next(i)) {
 			pmdb_t *db = alpm_list_getdata(i);
 
-			for(j = alpm_db_getpkgcache(db); j; j = alpm_list_next(j)) {
+			for(j = alpm_db_get_pkgcache(db); j; j = alpm_list_next(j)) {
 				dump_pkg_sync(alpm_list_getdata(j), alpm_db_get_name(db));
 			}
 		}
@@ -510,7 +512,7 @@ static int sync_list(alpm_list_t *syncs, alpm_list_t *targets)
 	for(i = ls; i; i = alpm_list_next(i)) {
 		pmdb_t *db = alpm_list_getdata(i);
 
-		for(j = alpm_db_getpkgcache(db); j; j = alpm_list_next(j)) {
+		for(j = alpm_db_get_pkgcache(db); j; j = alpm_list_next(j)) {
 			pmpkg_t *pkg = alpm_list_getdata(j);
 			if (!config->quiet) {
 				printf("%s %s %s\n", alpm_db_get_name(db), alpm_pkg_get_name(pkg),
@@ -561,7 +563,7 @@ static int sync_trans(alpm_list_t *targets)
 	if(config->op_s_upgrade) {
 		printf(_(":: Starting full system upgrade...\n"));
 		alpm_logaction("starting full system upgrade\n");
-		if(alpm_trans_sysupgrade() == -1) {
+		if(alpm_trans_sysupgrade(config->op_s_upgrade >= 2) == -1) {
 			pm_fprintf(stderr, PM_LOG_ERROR, "%s\n", alpm_strerrorlast());
 			retval = 1;
 			goto cleanup;
@@ -574,11 +576,12 @@ static int sync_trans(alpm_list_t *targets)
 			char *targ = alpm_list_getdata(i);
 			if(alpm_trans_addtarget(targ) == -1) {
 				pmgrp_t *grp = NULL;
-				int found=0;
+				int found = 0;
 				alpm_list_t *j;
 
-				if(pm_errno == PM_ERR_TRANS_DUP_TARGET) {
-					/* just ignore duplicate targets */
+				if(pm_errno == PM_ERR_TRANS_DUP_TARGET || pm_errno == PM_ERR_PKG_IGNORED) {
+					/* just skip duplicate or ignored targets */
+					pm_printf(PM_LOG_WARNING, _("skipping target: %s\n"), targ);
 					continue;
 				}
 				if(pm_errno != PM_ERR_PKG_NOT_FOUND) {
@@ -605,14 +608,14 @@ static int sync_trans(alpm_list_t *targets)
 									(char*)alpm_pkg_get_name(k->data));
 						}
 						list_display("   ", pkgnames);
-						if(yesno(1, _(":: Install whole content?"))) {
+						if(yesno(_(":: Install whole content?"))) {
 							for(k = pkgnames; k; k = alpm_list_next(k)) {
 								targets = alpm_list_add(targets, strdup(alpm_list_getdata(k)));
 							}
 						} else {
 							for(k = pkgnames; k; k = alpm_list_next(k)) {
 								char *pkgname = alpm_list_getdata(k);
-								if(yesno(1, _(":: Install %s from group %s?"), pkgname, targ)) {
+								if(yesno(_(":: Install %s from group %s?"), pkgname, targ)) {
 									targets = alpm_list_add(targets, strdup(pkgname));
 								}
 							}
@@ -640,7 +643,7 @@ static int sync_trans(alpm_list_t *targets)
 				for(i = data; i; i = alpm_list_next(i)) {
 					pmdepmissing_t *miss = alpm_list_getdata(i);
 					pmdepend_t *dep = alpm_miss_get_dep(miss);
-					char *depstring = alpm_dep_get_string(dep);
+					char *depstring = alpm_dep_compute_string(dep);
 					printf(_(":: %s: requires %s\n"), alpm_miss_get_target(miss),
 							depstring);
 					free(depstring);
@@ -667,23 +670,40 @@ static int sync_trans(alpm_list_t *targets)
 		goto cleanup;
 	}
 
-	if(!(alpm_trans_get_flags() & PM_TRANS_FLAG_PRINTURIS)) {
-		int confirm;
+	/* Step 3: actually perform the operation */
+	if(config->op_s_printuris) {
+		/* print uris */
+		alpm_list_t *i;
+		for(i = packages; i; i = alpm_list_next(i)) {
+			pmpkg_t *pkg = alpm_list_getdata(i);
+			pmdb_t *db = alpm_pkg_get_db(pkg);
+			const char *dburl = alpm_db_get_url(db);
+			if(dburl) {
+				printf("%s/%s\n", dburl, alpm_pkg_get_filename(pkg));
+			} else {
+				/* can't use WARNING here, we don't show warnings in -Sp... */
+				pm_fprintf(stderr, PM_LOG_ERROR, _("no database for package: %s\n"),
+						alpm_pkg_get_name(pkg));
+			}
 
-		display_synctargets(packages);
-		printf("\n");
-
-		if(config->op_s_downloadonly) {
-			confirm = yesno(1, _("Proceed with download?"));
-		} else {
-			confirm = yesno(1, _("Proceed with installation?"));
 		}
-		if(!confirm) {
-			goto cleanup;
-		}
-	}/* else 'print uris' requested.  We're done at this point */
+		/* we are done */
+		goto cleanup;
+	}
+
+	display_synctargets(packages);
+	printf("\n");
+
+	int confirm;
+	if(config->op_s_downloadonly) {
+		confirm = yesno(_("Proceed with download?"));
+	} else {
+		confirm = yesno(_("Proceed with installation?"));
+	}
+	if(!confirm) {
+		goto cleanup;
+	}
 
-	/* Step 3: actually perform the installation */
 	if(alpm_trans_commit(&data) == -1) {
 		pm_fprintf(stderr, PM_LOG_ERROR, _("failed to commit transaction (%s)\n"),
 		        alpm_strerrorlast());
@@ -740,7 +760,7 @@ int pacman_sync(alpm_list_t *targets)
 	alpm_list_t *sync_dbs = NULL;
 
 	/* Display only errors with -Sp and -Sw operations */
-	if(config->flags & (PM_TRANS_FLAG_DOWNLOADONLY | PM_TRANS_FLAG_PRINTURIS)) {
+	if((config->flags & PM_TRANS_FLAG_DOWNLOADONLY) || config->op_s_printuris) {
 		config->logmask &= ~PM_LOG_WARNING;
 	}
 
@@ -753,6 +773,7 @@ int pacman_sync(alpm_list_t *targets)
 		}
 
 		ret += sync_cleancache(config->op_s_clean);
+		printf("\n");
 		ret += sync_cleandb_all();
 
 		if(trans_release() == -1) {
@@ -812,13 +833,13 @@ int pacman_sync(alpm_list_t *targets)
 	}
 
 	alpm_list_t *targs = alpm_list_strdup(targets);
-	if(!(config->flags & (PM_TRANS_FLAG_DOWNLOADONLY | PM_TRANS_FLAG_PRINTURIS))) {
+	if(!(config->flags & PM_TRANS_FLAG_DOWNLOADONLY) && !config->op_s_printuris) {
 		/* check for newer versions of packages to be upgraded first */
 		alpm_list_t *packages = syncfirst();
 		if(packages) {
 			printf(_(":: The following packages should be upgraded first :\n"));
 			list_display("   ", packages);
-			if(yesno(1, _(":: Do you want to cancel the current operation\n"
+			if(yesno(_(":: Do you want to cancel the current operation\n"
 							":: and upgrade these packages now?"))) {
 				FREELIST(targs);
 				targs = packages;
