@@ -1,7 +1,8 @@
 /*
  *  util.c
  *
- *  Copyright (c) 2002-2007 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -69,19 +70,16 @@ int trans_release(void)
 	return(0);
 }
 
-int needs_transaction(void)
+int needs_root(void)
 {
-	if(config->op != PM_OP_MAIN && config->op != PM_OP_QUERY && config->op != PM_OP_DEPTEST) {
-		if((config->op == PM_OP_SYNC && !config->op_s_sync &&
-				(config->op_s_search || config->group || config->op_q_list || config->op_q_info))
-			 || config->op == PM_OP_DEPTEST) {
-			/* special case: PM_OP_SYNC can be used w/ config->op_s_search by any user */
-			return(0);
-		} else {
-			return(1);
-		}
+	if(config->op == PM_OP_UPGRADE || config->op == PM_OP_REMOVE || /* -U, -R */
+	   (config->op == PM_OP_SYNC && (config->op_s_clean || config->op_s_sync || /* -Sc, -Sy */
+	      (!config->group && !config->op_s_info && !config->op_q_list /* all other -S combinations, where */
+	        && !config->op_s_search && !config->op_s_printuris)))) {  /* -g, -i, -l, -s, -p is not set */
+		return(1);
+	} else {
+		return(0);
 	}
-	return(0);
 }
 
 /* gets the current screen column width */
@@ -579,7 +577,7 @@ void display_targets(const alpm_list_t *pkgs, int install)
 }
 
 /* Display a list of transaction targets.
- * `pkgs` should be a list of pmsyncpkg_t's,
+ * `pkgs` should be a list of pmpkg_t's,
  * retrieved from a transaction object
  */
 void display_synctargets(const alpm_list_t *syncpkgs)
@@ -588,13 +586,12 @@ void display_synctargets(const alpm_list_t *syncpkgs)
 	alpm_list_t *pkglist = NULL, *rpkglist = NULL;
 
 	for(i = syncpkgs; i; i = alpm_list_next(i)) {
-		pmsyncpkg_t *sync = alpm_list_getdata(i);
-		pmpkg_t *pkg = alpm_sync_get_pkg(sync);
+		pmpkg_t *pkg = alpm_list_getdata(i);
 		pkglist = alpm_list_add(pkglist, pkg);
 
 		/* The removes member contains a list of packages to be removed
 		 * due to the package that is being installed. */
-		alpm_list_t *to_replace = alpm_sync_get_removes(sync);
+		alpm_list_t *to_replace = alpm_pkg_get_removes(pkg);
 
 		for(j = to_replace; j; j = alpm_list_next(j)) {
 			pmpkg_t *rp = alpm_list_getdata(j);
@@ -610,6 +607,25 @@ void display_synctargets(const alpm_list_t *syncpkgs)
 	alpm_list_free(rpkglist);
 }
 
+/* Helper function for comparing strings using the
+ * alpm "compare func" signature */
+int str_cmp(const void *s1, const void *s2)
+{
+	return(strcmp(s1, s2));
+}
+
+void display_new_optdepends(pmpkg_t *oldpkg, pmpkg_t *newpkg)
+{
+	alpm_list_t *old = alpm_pkg_get_optdepends(oldpkg);
+	alpm_list_t *new = alpm_pkg_get_optdepends(newpkg);
+	alpm_list_t *optdeps = alpm_list_diff(new,old,str_cmp);
+	if(optdeps) {
+		printf(_("New optional dependencies for %s\n"), alpm_pkg_get_name(newpkg));
+		list_display_linebreak("   ", optdeps);
+	}
+	alpm_list_free(optdeps);
+}
+
 void display_optdepends(pmpkg_t *pkg)
 {
 	alpm_list_t *optdeps = alpm_pkg_get_optdepends(pkg);
@@ -620,10 +636,9 @@ void display_optdepends(pmpkg_t *pkg)
 }
 
 /* presents a prompt and gets a Y/N answer */
-int yesno(short preset, char *fmt, ...)
+static int question(short preset, char *fmt, va_list args)
 {
 	char response[32];
-	va_list args;
 	FILE *stream;
 
 	if(config->noconfirm) {
@@ -633,9 +648,7 @@ int yesno(short preset, char *fmt, ...)
 		stream = stderr;
 	}
 
-	va_start(args, fmt);
 	vfprintf(stream, fmt, args);
-	va_end(args);
 
 	if(preset) {
 		fprintf(stream, " %s ", _("[Y/n]"));
@@ -663,6 +676,30 @@ int yesno(short preset, char *fmt, ...)
 	return(0);
 }
 
+int yesno(char *fmt, ...)
+{
+	int ret;
+	va_list args;
+
+	va_start(args, fmt);
+	ret = question(1, fmt, args);
+	va_end(args);
+
+	return(ret);
+}
+
+int noyes(char *fmt, ...)
+{
+	int ret;
+	va_list args;
+
+	va_start(args, fmt);
+	ret = question(0, fmt, args);
+	va_end(args);
+
+	return(ret);
+}
+
 int pm_printf(pmloglevel_t level, const char *format, ...)
 {
 	int ret;
