@@ -24,7 +24,6 @@
 #include <limits.h>
 #include <string.h>
 #include <dirent.h>
-#include <libgen.h>
 
 #include <alpm.h>
 #include <alpm_list.h>
@@ -56,7 +55,7 @@ void output_cb(pmloglevel_t level, char *fmt, va_list args)
 	}
 }
 
-static int db_test(char *dbpath)
+static int db_test(char *dbpath, int local)
 {
 	struct dirent *ent;
 	char path[PATH_MAX];
@@ -70,7 +69,8 @@ static int db_test(char *dbpath)
 	}
 
 	while ((ent = readdir(dir)) != NULL) {
-		if(!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, "..")) {
+		if(!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, "..")
+				|| ent->d_name[0] == '.') {
 			continue;
 		}
 		/* check for desc, depends, and files */
@@ -84,36 +84,133 @@ static int db_test(char *dbpath)
 			printf("%s: dependency file is missing\n", ent->d_name);
 			ret++;
 		}
-		snprintf(path, PATH_MAX, "%s/%s/files", dbpath, ent->d_name);
-		if(access(path, F_OK)) {
-			printf("%s: file list is missing\n", ent->d_name);
-			ret++;
+		if(local) {
+			snprintf(path, PATH_MAX, "%s/%s/files", dbpath, ent->d_name);
+			if(access(path, F_OK)) {
+				printf("%s: file list is missing\n", ent->d_name);
+				ret++;
+			}
 		}
 	}
 	return(ret);
 }
 
-int main(int argc, char **argv)
+int checkdeps(alpm_list_t *pkglist)
 {
-	int retval = 0; /* default = false */
-	pmdb_t *db = NULL;
-	char *dbpath;
-	char localdbpath[PATH_MAX];
-	alpm_list_t *i;
-
-	if(argc == 1) {
-		dbpath = DBPATH;
-	} else if(argc == 3 && strcmp(argv[1], "-b") == 0) {
-		dbpath = argv[2];
-	} else {
-		fprintf(stderr, "usage: %s -b <pacman db>\n", BASENAME);
-		return(1);
+	alpm_list_t *data, *i;
+	int ret = 0;
+	/* check dependencies */
+	data = alpm_checkdeps(pkglist, 0, NULL, pkglist);
+	for(i = data; i; i = alpm_list_next(i)) {
+		pmdepmissing_t *miss = alpm_list_getdata(i);
+		pmdepend_t *dep = alpm_miss_get_dep(miss);
+		char *depstring = alpm_dep_compute_string(dep);
+		printf("missing dependency for %s : %s\n", alpm_miss_get_target(miss),
+				depstring);
+		free(depstring);
+		ret++;
 	}
+	return(ret);
+}
+
+int checkconflicts(alpm_list_t *pkglist)
+{
+	alpm_list_t *data, *i;
+	int ret = 0;
+	/* check conflicts */
+	data = alpm_checkconflicts(pkglist);
+	for(i = data; i; i = i->next) {
+		pmconflict_t *conflict = alpm_list_getdata(i);
+		printf("%s conflicts with %s\n", alpm_conflict_get_package1(conflict),
+				alpm_conflict_get_package2(conflict));
+		ret++;
+	}
+	return(ret);
+}
+
+int check_localdb(char *dbpath) {
+	char localdbpath[PATH_MAX];
+	int ret = 0;
+	pmdb_t *db = NULL;
+	alpm_list_t *pkglist;
 
 	snprintf(localdbpath, PATH_MAX, "%s/local", dbpath);
-	retval = db_test(localdbpath);
-	if(retval) {
-		return(retval);
+	ret = db_test(localdbpath, 1);
+	if(ret) {
+		return(ret);
+	}
+
+	db = alpm_db_register_local();
+	if(db == NULL) {
+		fprintf(stderr, "error: could not register 'local' database (%s)\n",
+				alpm_strerrorlast());
+		cleanup(EXIT_FAILURE);
+	}
+	pkglist = alpm_db_get_pkgcache(db);
+	ret += checkdeps(pkglist);
+	ret += checkconflicts(pkglist);
+	return(ret);
+}
+
+int check_syncdbs(char *dbpath, alpm_list_t *dbnames) {
+	char syncdbpath[PATH_MAX];
+	int ret = 0;
+	pmdb_t *db = NULL;
+	alpm_list_t *i, *pkglist, *syncpkglist = NULL;
+
+	for(i = dbnames; i; i = alpm_list_next(i)) {
+		char *dbname = alpm_list_getdata(i);
+		snprintf(syncdbpath, PATH_MAX, "%s/sync/%s", dbpath, dbname);
+		ret = db_test(syncdbpath, 0);
+		if(ret) {
+			return(ret);
+		}
+		db = alpm_db_register_sync(dbname);
+		if(db == NULL) {
+			fprintf(stderr, "error: could not register sync database (%s)\n",
+					alpm_strerrorlast());
+			cleanup(EXIT_FAILURE);
+		}
+		pkglist = alpm_db_get_pkgcache(db);
+		syncpkglist = alpm_list_join(syncpkglist, alpm_list_copy(pkglist));
+	}
+	ret += checkdeps(syncpkglist);
+	alpm_list_free(syncpkglist);
+
+	alpm_db_unregister_all();
+	return(ret);
+}
+
+void usage() {
+	fprintf(stderr, "usage:\n");
+  fprintf(stderr,
+			"\t%s [-b <pacman db>]                : check the local database\n", BASENAME);
+  fprintf(stderr,
+			"\t%s [-b <pacman db>] core extra ... : check the listed sync databases\n", BASENAME);
+	exit(1);
+}
+
+int main(int argc, char **argv)
+{
+	int ret = 0;
+	char *dbpath = DBPATH;
+	int a = 1;
+	alpm_list_t *dbnames = NULL;
+
+	while(a < argc) {
+		if(strcmp(argv[a], "-b") == 0) {
+			if(++a < argc) {
+				dbpath = argv[a];
+			} else {
+				usage();
+			}
+		}	else if(strcmp(argv[a], "-h") == 0 ||
+				strcmp(argv[a], "--help") == 0 ) {
+			usage();
+		} else {
+			dbnames = alpm_list_add(dbnames, argv[a]);
+		}
+		a++;
 	}
 
 	if(alpm_initialize() == -1) {
@@ -126,34 +223,15 @@ int main(int argc, char **argv)
 
 	alpm_option_set_dbpath(dbpath);
 
-	db = alpm_db_register_local();
-	if(db == NULL) {
-		fprintf(stderr, "error: could not register 'local' database (%s)\n",
-				alpm_strerrorlast());
-		cleanup(EXIT_FAILURE);
+	if(!dbnames) {
+		printf("Checking the integrity of the local database in %s\n",dbpath);
+		ret = check_localdb(dbpath);
+	} else {
+		printf("Checking the integrity of the sync databases in %s\n",dbpath);
+		ret = check_syncdbs(dbpath,dbnames);
 	}
 
-	/* check dependencies */
-	alpm_list_t *data;
-	data = alpm_checkdeps(db, 0, NULL, alpm_db_getpkgcache(db));
-	for(i = data; i; i = alpm_list_next(i)) {
-		pmdepmissing_t *miss = alpm_list_getdata(i);
-		pmdepend_t *dep = alpm_miss_get_dep(miss);
-		char *depstring = alpm_dep_get_string(dep);
-		printf("missing dependency for %s : %s\n", alpm_miss_get_target(miss),
-				depstring);
-		free(depstring);
-	}
-
-	/* check conflicts */
-	data = alpm_checkdbconflicts(db);
-	for(i = data; i; i = i->next) {
-		pmconflict_t *conflict = alpm_list_getdata(i);
-		printf("%s conflicts with %s\n", alpm_conflict_get_package1(conflict),
-				alpm_conflict_get_package2(conflict));
-	}
-
-	cleanup(retval);
+	cleanup(ret);
 }
 
 /* vim: set ts=2 sw=2 noet: */
